@@ -14,7 +14,7 @@ from kvlayer._abstract_storage import AbstractStorage
 from pyaccumulo import Accumulo, Mutation, Range, BatchWriter
 from pyaccumulo.iterators import RowDeletingIterator
 from pyaccumulo.proxy.ttypes import IteratorScope, AccumuloSecurityException
-from _utils import join_uuids, split_uuids
+from _utils import split_key, make_start_key, make_end_key, join_key_fragments
 
 logger = logging.getLogger('kvlayer')
 
@@ -24,11 +24,11 @@ class AStorage(AbstractStorage):
     Accumulo storage implements kvlayer's AbstractStorage, which
     manages a set of tables as specified to setup_namespace
     '''
-    def __init__(self, config):
-        super(AStorage, self).__init__(config)
+    def __init__(self):
+        super(AStorage, self).__init__()
 
         self._connected = False
-        addresses = config.get('storage_addresses', [])
+        addresses = self._config.get('storage_addresses', [])
         if not addresses:
             raise ProgrammerError('config lacks storage_addresses')
 
@@ -44,27 +44,27 @@ class AStorage(AbstractStorage):
 
         ## The following are all parameters to the accumulo
         ## batch interfaces.
-        self._max_memory = config.get('accumulo_max_memory', 1000000)
-        self._timeout_ms = config.get('accumulo_timeout_ms', 30000)
-        self._threads = config.get('accumulo_threads', 10)
-        self._latency_ms = config.get('accumulo_latency_ms', 10)
+        self._max_memory = self._config.get('accumulo_max_memory', 1000000)
+        self._timeout_ms = self._config.get('accumulo_timeout_ms', 30000)
+        self._threads = self._config.get('accumulo_threads', 10)
+        self._latency_ms = self._config.get('accumulo_latency_ms', 10)
 
         ## Acumulo storage requires a username and password
-        self._user = config.get('username', None)
-        self._password = config.get('password', None)
+        self._user = self._config.get('username', None)
+        self._password = self._config.get('password', None)
         if not self._user and self._password:
             raise ProgrammerError('accumulo storage requires '
                                   'username/password')
 
         self.thrift_framed_transport_size_in_mb = \
-            config['thrift_framed_transport_size_in_mb']
+            self._config['thrift_framed_transport_size_in_mb']
 
         self._conn = None
 
     @property
     def conn(self):
         if not self._conn:
-            logger.critical('connecting to Accumulo')
+            logger.info('connecting to Accumulo')
             self._conn = Accumulo(self._host, self._port,
                                   self._user, self._password)
             self._connected = True
@@ -103,6 +103,7 @@ class AStorage(AbstractStorage):
         '''
         logger.debug('creating tables: %r', table_names)
         self._table_names.update(table_names)
+        self.normalize_namespaces(self._table_names)
         for table in table_names:
             if not self.conn.table_exists(self._ns(table)):
                 self._create_table(table)
@@ -125,7 +126,7 @@ class AStorage(AbstractStorage):
 
     @retry([AccumuloSecurityException])
     def put(self, table_name, *keys_and_values, **kwargs):
-        num_uuids = self._table_names[table_name]
+        key_spec = self._table_names[table_name]
         cur_bytes = 0
         batch_writer = BatchWriter(conn=self.conn,
                                    table=self._ns(table_name),
@@ -134,7 +135,7 @@ class AStorage(AbstractStorage):
                                    timeout_ms=self._timeout_ms,
                                    threads=self._threads)
         for key, blob in keys_and_values:
-            ex = self.check_put_key_value(key, blob, table_name, num_uuids)
+            ex = self.check_put_key_value(key, blob, table_name, key_spec)
             if ex:
                 raise ex
             if (len(blob) + cur_bytes >=
@@ -147,28 +148,39 @@ class AStorage(AbstractStorage):
                              'batched, and will send this item in next batch.')
                 batch_writer.flush()
                 cur_bytes = 0
-            mut = Mutation(join_uuids(*key))
+            mut = Mutation(join_key_fragments(key, key_spec=key_spec))
             mut.put(cf='', cq='', val=blob)
             batch_writer.add_mutation(mut)
             cur_bytes += len(blob)
         batch_writer.close()
 
     def scan(self, table_name, *key_ranges, **kwargs):
-        num_uuids = self._table_names[table_name]
+        key_spec = self._table_names[table_name]
         if not key_ranges:
             key_ranges = [['', '']]
         for start_key, stop_key in key_ranges:
             total_count = 0
             specific_key_range = bool(start_key or stop_key)
             if specific_key_range:
-                joined_key = join_uuids(*start_key, num_uuids=num_uuids,
-                                        padding='0')
-                ## If there is no start key we need to scan from the beginning of the range.  To include any value
-                ## stored at UUID(int=0) we need a key that occurs before the key with all zeros.  The key we use
-                ## to accomplish this is '.' just like the _preceeding_key function.
-                srow = len(start_key)>0 and self._preceeding_key(joined_key) or '.'
-                erow = len(stop_key)>0 and join_uuids(*stop_key, num_uuids=num_uuids, padding='f') or 'f' * 32 * num_uuids
-                key_range = Range(srow=srow, erow=erow)
+                # Accumulo treats None as a negative-infinity or
+                # positive-infinity key as needed for starts and ends
+                # of ranges.
+                #
+                # pyaccumulo has a bug at present 20140228_171555
+                # which causes it to never do a '>=' scan, so we must
+                # decrement the start key to include the start key
+                # which necessary for a get() operation.
+                # https://github.com/accumulo/pyaccumulo/issues/14
+                if not start_key:
+                    srow = None
+                else:
+                    srow = make_start_key(start_key, key_spec=key_spec)
+                    srow = _string_decrement(srow)
+                if not stop_key:
+                    erow = None
+                else:
+                    erow = make_end_key(stop_key, key_spec=key_spec)
+                key_range = Range(srow=srow, erow=erow, sinclude=True, einclude=True)
                 scanner = self.conn.scan(self._ns(table_name),
                                          scanrange=key_range)
             else:
@@ -176,7 +188,7 @@ class AStorage(AbstractStorage):
 
             for row in scanner:
                 total_count += 1
-                yield tuple(split_uuids(row.row)), row.val
+                yield split_key(row.row, key_spec), row.val
             else:
                 if specific_key_range and total_count == 0:
                     raise MissingID('table_name=%r start=%r finish=%r' % (
@@ -185,7 +197,18 @@ class AStorage(AbstractStorage):
     def get(self, table_name, *keys, **kwargs):
         for key in keys:
             gen = self.scan(table_name, (key, key))
-            yield gen.next()
+            for k, v in gen:
+                if k == key:
+                    yield k, v
+                else:
+                    # This is only a warning because the scan might
+                    # not be precise and could, unfortunately,
+                    # legitimately yield something one key plus or
+                    # minus the key we actually want.
+                    #
+                    # When pyaccumulo gets fixed for start-inclusive
+                    # scan, this might go away.
+                    logger.warn('got key %r while get()ing key %r', k, key)
 
     def close(self):
         self._connected = False
@@ -195,13 +218,6 @@ class AStorage(AbstractStorage):
     def __del__(self):
         self.close()
 
-    def _preceeding_key(self, key):
-        num = (int(key, 16) - 1)
-        if num < 0:
-            return '.'
-        format_string = '%%0.%dx' % len(key)
-        return format_string % num
-
     def delete(self, table_name, *keys, **kwargs):
         batch_writer = BatchWriter(conn=self.conn,
                                    table=self._ns(table_name),
@@ -210,7 +226,21 @@ class AStorage(AbstractStorage):
                                    timeout_ms=self._timeout_ms,
                                    threads=self._threads)
         for key in keys:
-            mut = Mutation(join_uuids(*key))
+            mut = Mutation(join_key_fragments(key, key_spec=self._table_names[table_name]))
             mut.put(cf='', cq='', val='DEL_ROW')
             batch_writer.add_mutation(mut)
         batch_writer.close()
+
+
+def _string_decrement(x):
+    if len(x) < 1:
+        return None  # None is before all keys, aka negative infinity
+    pre = x[:-1]
+    post = ord(x[-1])
+    if post > 0:
+        return pre + chr(post - 1)
+    else:
+        pre = _string_decrement(pre)
+        if pre is None:
+            return None
+        return pre + '\xff'
