@@ -39,3 +39,7 @@ class BadKey(KVLayerError):
 
     """
     pass
+
+class SerializationError(KVLayerError):
+    """Converting between an item and a serialized form failed."""
+    pass
