@@ -11,7 +11,7 @@ import abc
 import logging
 from kvlayer._exceptions import MissingID
 from kvlayer._abstract_storage import AbstractStorage
-from kvlayer._utils import _requires_connection, join_uuids
+from kvlayer._utils import _requires_connection, make_start_key, make_end_key, join_key_fragments
 
 logger = logging.getLogger(__name__)
 
@@ -23,9 +23,10 @@ class AbstractLocalStorage(AbstractStorage):
 
     """
 
-    def __init__(self, config):
-        super(AbstractLocalStorage, self).__init__(config)
+    def __init__(self):
+        super(AbstractLocalStorage, self).__init__()
         self._connected = False
+        self._raise_on_missing = self._config.get('raise_on_missing', True)
 
     def setup_namespace(self, table_names):
         '''creates tables in the namespace.  Can be run multiple times with
@@ -34,6 +35,7 @@ class AbstractLocalStorage(AbstractStorage):
         '''
         logger.debug('creating tables: %r', table_names)
         self._table_names.update(table_names)
+        self.normalize_namespaces(self._table_names)
         ## just store everything in a dict
         for table in table_names:
             if table not in self._data:
@@ -56,36 +58,42 @@ class AbstractLocalStorage(AbstractStorage):
 
     @_requires_connection
     def scan(self, table_name, *key_ranges, **kwargs):
-        num_uuids = self._table_names[table_name]
+        key_spec = self._table_names[table_name]
         key_ranges = list(key_ranges)
         specific_key_range = True
         if not key_ranges:
-            key_ranges = [[('',), ('',)]]
+            key_ranges = [[None, None]]
             specific_key_range = False
         for start, finish in key_ranges:
             total_count = 0
-            start = len(start)>0  and join_uuids(*start,  num_uuids=num_uuids, padding='0') or '0' * 32 * num_uuids
-            finish = len(finish)>0 and join_uuids(*finish, num_uuids=num_uuids, padding='f') or 'f' * 32 * num_uuids
+            start = make_start_key(start, key_spec=key_spec)
+            finish = make_end_key(finish, key_spec=key_spec)
             for key in sorted(self._data[table_name].iterkeys()):
                 ## given a range, mimic the behavior of DBs that tell
                 ## you if they failed to find a key
-                joined_key = join_uuids(*key)
-                if start <= joined_key <= finish:
-                    total_count += 1
-                    yield key, self._data[table_name][key]
+                ## LocalStorage does get/put on the Python tuple as the key, stringify for sort comparison
+                joined_key = join_key_fragments(key, key_spec=key_spec)
+                if (start is not None) and (start > joined_key):
+                    continue
+                if (finish is not None) and (finish < joined_key):
+                    continue
+                total_count += 1
+                yield key, self._data[table_name][key]
             else:
                 if specific_key_range and total_count == 0:
                     ## specified a key range, but found none
-                    raise MissingID()
+                    if self._raise_on_missing:
+                        raise MissingID()
 
     @_requires_connection
     def get(self, table_name, *keys, **kwargs):
         for key in keys:
             try:
                 key, value = key, self._data[table_name][key]
+                yield key, value
             except KeyError:
-                raise MissingID('table_name=%r key: %r' % ( table_name, key))
-            yield key, value
+                if self._raise_on_missing:
+                    raise MissingID('table_name=%r key: %r' % ( table_name, key))
 
 
     @_requires_connection
@@ -105,8 +113,5 @@ class LocalStorage(AbstractLocalStorage):
 
     _data = {}
 
-    def __init__(self, config):
-        super(LocalStorage, self).__init__(config)
-
     def delete_namespace(self):
         self._data.clear()
