@@ -22,7 +22,7 @@ kvlayer "table" is a hash stored in a single row, where the keys of
 the hash are UUID tuples.
 
 This currently uses pure-ASCII key values (using
-:func:`kvlayer._utils.join_uuids`) and intermediate table row IDs.
+:func:`kvlayer._utils.join_key_fragments`) and intermediate table row IDs.
 Redis should in principle be able to handle packed-binary values,
 which would be a quarter the size.
 
@@ -46,16 +46,15 @@ import redis
 
 from kvlayer._abstract_storage import AbstractStorage
 from kvlayer._exceptions import BadKey, MissingID, ProgrammerError
-from kvlayer._utils import join_uuids, split_uuids
+from kvlayer._utils import join_key_fragments, split_key, make_start_key, make_end_key
 
 logger = logging.getLogger(__name__)
 
 class RedisStorage(AbstractStorage):
-    def __init__(self, config):
+    def __init__(self):
         """Initialize a redis-based storage instance.
 
-        `config` is a dictionary that may include the following
-        configuration parameters:
+        Uses the global kvlayer configuration, with the following parameters:
 
         ``namespace``
           namespace prefix for this storage layer
@@ -63,11 +62,13 @@ class RedisStorage(AbstractStorage):
           application name prefix for this storage layer
         ``storage_addresses``
           list of ``hostname:port`` pairs for redis (only first is used)
+        ``redis_db_num``
+          Redis database number (defaults to 0)
 
         """
-        super(RedisStorage, self).__init__(config)
-        storage_addresses = config.get('storage_addresses', [])
-        db_num = config.get('redis_db_num', 0)
+        super(RedisStorage, self).__init__()
+        storage_addresses = self._config.get('storage_addresses', [])
+        db_num = self._config.get('redis_db_num', 0)
         if len(storage_addresses) == 0:
             raise ProgrammerError('config lacks storage_addresses')
         if len(storage_addresses) > 1:
@@ -109,7 +110,7 @@ class RedisStorage(AbstractStorage):
         """
         if table not in self._table_keys:
             k = conn.hget(self._namespace_key, table)
-            if k is not none: self._table_keys[table] = k
+            if k is not None: self._table_keys[table] = k
         return self._table_keys.get(table, None)
 
     def setup_namespace(self, table_names):
@@ -159,6 +160,7 @@ class RedisStorage(AbstractStorage):
                         raise
                     tries -= 1
                     pass # try again with a new uuid
+        self.normalize_namespaces(self._table_sizes)
         return
 
     def delete_namespace(self):
@@ -227,11 +229,12 @@ class RedisStorage(AbstractStorage):
         params = []
         for (k,v) in keys_and_values:
             #logger.debug('put {} {!r} {}'.format(table_name, k, v))
+            key_spec = self._table_sizes[table_name]
             ex = self.check_put_key_value(k, v, table_name,
-                                          self._table_sizes[table_name])
+                                          key_spec)
             if ex is not None:
                 raise ex
-            params.append(join_uuids(*k))
+            params.append(join_key_fragments(k, key_spec=key_spec))
             params.append(v)
         script = conn.register_script("""
         if redis.call('exists', KEYS[1]) == 0
@@ -283,10 +286,11 @@ class RedisStorage(AbstractStorage):
         #
         # Until that works, let's just get a dump of everything in
         # the hash and hope there's not too much there.
-        num_uuids = self._table_sizes[table_name]
-        ranges = [(join_uuids(*l, num_uuids=num_uuids, padding='0'),
-                   join_uuids(*u, num_uuids=num_uuids, padding='f'))
-                  for (l,u) in key_ranges]
+        key_spec = self._table_sizes[table_name]
+        ranges = [
+            (make_start_key(l, key_spec=key_spec),
+             make_end_key(u, key_spec=key_spec))
+            for (l,u) in key_ranges]
         def valid(k):
             if k == '': return False # placeholder
             if len(ranges) == 0: return True
@@ -298,7 +302,7 @@ class RedisStorage(AbstractStorage):
         for k in sorted(res.iterkeys()):
             if valid(k):
                 found = True
-                uuids = split_uuids(k)
+                uuids = split_key(k, key_spec)
                 yield (uuids,res[k])
         if len(ranges) > 0 and not found:
             raise MissingID(key_ranges)
@@ -333,14 +337,15 @@ class RedisStorage(AbstractStorage):
         key = self._table_key(conn, table_name)
         if key is None:
             raise BadKey(key)
+        key_spec = self._table_sizes[table_name]
         #logger.debug('get {} {!r}'.format(table_name, keys))
-        ks = [join_uuids(*k) for k in keys]
+        ks = [join_key_fragments(k, key_spec=key_spec) for k in keys]
         vs = conn.hmget(key, *ks)
         found = False
         for (k, v) in zip(ks, vs):
             if v is not None:
                 found = True
-                yield (tuple(split_uuids(k)),v)
+                yield (tuple(split_key(k, key_spec)),v)
         if not found:
             raise MissingID(keys)
             
@@ -371,7 +376,8 @@ class RedisStorage(AbstractStorage):
         if key is None:
             raise BadKey(key)
         logger.debug('delete {} {!r}'.format(table_name, keys))
-        ks = [join_uuids(*k) for k in keys]
+        key_spec = self._table_sizes[table_name]
+        ks = [join_key_fragments(k, key_spec=key_spec) for k in keys]
         conn.hdel(key, *ks)
 
     def close(self):
