@@ -5,10 +5,11 @@ Your use of this software is governed by your license agreement.
 Copyright 2012-2014 Diffeo, Inc.
 
 """
-
+from __future__ import absolute_import
 import errno
 import logging
 import os
+import pdb
 import random
 import sys
 import time
@@ -18,12 +19,16 @@ import py
 import pytest
 import yaml
 
+from pytest_diffeo import redis_address
 import kvlayer
 from kvlayer import MissingID, BadKey
+import yakonfig
 
 logger = logging.getLogger(__name__)
 
-backends = ['local', 'filestorage', 'cassandra', 'accumulo', 'redis']
+# TODO: fix cassandra for new flexible key-tuples
+#backends = ['local', 'filestorage', 'cassandra', 'accumulo', 'redis']
+backends = ['local', 'filestorage', 'redis', 'accumulo']
 try:
     import psycopg2
     backends.append('postgres')
@@ -36,50 +41,36 @@ except ImportError, e:
 def backend(request):
     return request.param
 
-@pytest.fixture(scope='module')
-def config(backend, request):
-    config_path = request.fspath.new(basename='config_{}.yaml'.format(backend))
-    try:
-        with config_path.open('r') as f:
-            return yaml.load(f)
-    except py.error.ENOENT:
-        return { 'storage_type': backend,
-                 'namespace': None,
-                 'storage_addresses': None }
-
-@pytest.fixture(scope='function')
-def client(config, request, tmpdir, namespace_string):
-    config['namespace'] = namespace_string
-    config['app_name'] = 'kvlayer'
+@pytest.yield_fixture(scope='function')
+def client(backend, request, tmpdir, namespace_string):
+    config_path = str(request.fspath.dirpath('config_{}.yaml'.format(backend)))
+    params = dict(
+        app_name='kvlayer',
+        namespace=namespace_string,
+    )
 
     # this is hacky but must go somewhere
-    if config['storage_type'] == 'filestorage':
+    if backend == 'filestorage':
         local = tmpdir.join('local')
         with local.open('w') as f: pass
-        config['filename'] = str(local)
+        params['kvlayer_filename'] = str(local)
 
-        copy = tmpdir.join('copy')
-        config['copy_to_filename'] = str(copy)
+    if backend == 'redis':
+        params['storage_addresses'] = [ redis_address(request) ]
 
-    if config['storage_type'] == 'redis':
-        redis_address = request.config.getoption('--redis-address')
-        if redis_address is not None:
-            config['storage_addresses'] = [redis_address]
-
-    client = kvlayer.client(config)
-    client.delete_namespace()
-
-    def fin():
+    with yakonfig.defaulted_config([kvlayer], filename=config_path,
+                                   params=params):
+        client = kvlayer.client()
+        client.delete_namespace()
+        yield client
         client.delete_namespace()
-    request.addfinalizer(fin)
-
-    return client
-
 
 def test_basic_storage(client):
     client.setup_namespace(dict(t1=2, t2=3))
+    assert 0 == len(list(client.scan('t1')))
     # use time-based UUID 1, so these are ordered
     u1, u2, u3 = uuid.uuid1(), uuid.uuid1(), uuid.uuid1()
+    #pdb.set_trace()
     client.put('t1', ((u1, u2), b'88'))
     client.put('t2', ((u1, u2, u3), b'88'))
     assert 1 == len(list(client.scan('t1')))
@@ -325,3 +316,49 @@ def test_no_keys(client):
 
     client.put('t1')
     assert list(client.scan('t1')) in [ [(u, 'value')], [(uu, 'value')] ]
+
+
+def test_new_key_spec(client):
+    """Test that new table key specs work in setup_namespace."""
+    client.setup_namespace({
+        'kt2': (str,int),
+        'kt3': (uuid.UUID, (int,long)),
+        'kt4': (str,str,str),
+    })
+
+    good_kt2_key = ('aoeu', 1337)
+    client.put('kt2', (good_kt2_key, 'blah'))
+
+    with pytest.raises(BadKey):
+        client.put('kt2', ((147, 'aeou'), 'nope'))
+    with pytest.raises(BadKey):
+        client.put('kt2', ((uuid.uuid4(), 'aeou'), 'nope'))
+    with pytest.raises(BadKey):
+        client.put('kt2', (('foo', 'aeou'), 'nope'))
+    with pytest.raises(BadKey):
+        client.put('kt2', (('foo', 123987419234701239847120), 'nope'))
+
+    kt3u = uuid.uuid4()
+    good_kt3_kvs = [
+        ((kt3u, 42), 'v1'),
+        ((kt3u, 123987419234701239847120), 'v2'),
+    ]
+    client.put('kt3', *good_kt3_kvs)
+
+    count = 0
+    for k,v in client.scan('kt3', ((kt3u,), (kt3u,))):
+        assert (k,v) in good_kt3_kvs
+        count += 1
+    assert count == len(good_kt3_kvs)
+
+    good_kt4_kvs = [
+        (('fooa', 'aoeu', 'snth'), 'v1'),
+        (('foob', 'aoeu', 'snth'), 'v1'),
+        (('fooc', 'aoeu', 'snth'), 'v1'),
+    ]
+    client.put('kt4', *good_kt4_kvs)
+    count = 0
+    for k,v in client.scan('kt4', (('foo',), ('foo',))):
+        assert (k,v) in good_kt4_kvs
+        count += 1
+    assert count == len(good_kt4_kvs)
